<?php

/**
 * Implements hook_drush_command().
 *
 * @return array
 */
function browser_refresh_drush_command() {
  $items['browser-refresh-start'] = array(
    'description' => 'Start browser refresh and keep it running.',
    'aliases' => array('brstart'),
  );
  $items['browser-refresh-stop'] = array(
    'description' => 'Stop browser refresh.',
    'aliases' => array('brstop'),
  );
  $items['browser-refresh-status'] = array(
    'description' => 'Get the status of browser refresh.',
    'aliases' => array('brstatus'),
  );
  $items['browser-refresh-restart'] = array(
    'description' => 'Stop and start browser refresh and keep it running.',
    'aliases' => array('brrestart'),
  );

  return $items;
}

function _browser_refresh_drush_exec($command) {
  chdir(DRUPAL_ROOT . '/' . drupal_get_path('module', 'browser_refresh'));

  $op = array();
  exec($command, $op);
  return $op[0];
}

function drush_browser_refresh_start() {

  // Check if browser-refresh is already running
  if ($pid = browser_refresh_status()) {
    drupal_set_message('Browser Refresh already started.', 'warning');
    return;
  }

  // Check if browser-refresh is installed
  $executable = _browser_refresh_drush_exec('which browser-refresh');
  if (empty($executable)) {
    drupal_set_message(t('Main application browser-refresh is not available!'), 'error');
    drupal_set_message(t('Try calling this: npm install -g browser-refresh'), 'status');
    return;
  }

  // Check if browser-refresh-client is installed
  if (_browser_refresh_drush_exec('node br.js check') != 'OK!') {
    drupal_set_message(t('Required node modules are not installed!'), 'error');
    drupal_set_message(t('Try calling this: npm install -g browser-refresh-client'), 'status');
    return;
  }

  // Write configuration for the new process.
  $config_filename = __DIR__ . '/.browser-refresh';
  $temp_filename = drupal_realpath(drupal_tempnam('temporary://', 'browser-refresh-'));
  $config = array(
    'watch' => array(DRUPAL_ROOT),
    'urlFileName' => $temp_filename,
  );
  file_put_contents($config_filename, json_encode($config));

  // Start the process.
  $pid = (int) _browser_refresh_drush_exec('nohup browser-refresh > /dev/null 2>&1 & echo $!');
  if ($pid && file_exists($temp_filename)) {
    $url = '';
    while (empty($url)) {
      sleep(1);
      $url = file_get_contents($temp_filename);
    }
    variable_set('browser_refresh_url', $url);
    variable_set('browser_refresh_pid', $pid);
  }
  unlink($config_filename);
  unlink($temp_filename);

  drush_browser_refresh_status();
  drupal_set_message('You should refresh your browser to see the updated status.');
}

function drush_browser_refresh_stop() {
  if ($pid = browser_refresh_status()) {
    variable_set('browser_refresh_pid', 0);
    exec('kill ' . $pid);
    drupal_set_message('Browser Refresh stopped.');
    drupal_set_message('You should refresh your browser to see the updated status.');
  }
  else {
    drupal_set_message('Browser Refresh is not running.', 'error');
  }
}

function drush_browser_refresh_status() {
  if ($pid = browser_refresh_status()) {
    drupal_set_message(t('Browser Refresh is running (pid=!pid).', array('!pid' => $pid)));
  }
  else {
    drupal_set_message('Browser Refresh is not running.', 'error');
  }
}

function drush_browser_refresh_restart() {
  drush_browser_refresh_stop();
  drush_browser_refresh_start();
}
